@@ -4,6 +4,7 @@ Convert between sequence formats
 import argparse
 import functools
 import logging
+import random
 
 from Bio import Alphabet, SeqIO
 from Bio.Alphabet import IUPAC
@@ -36,6 +37,9 @@ def add_options(parser):
         help='Perform sorting by length or name, ascending or descending. '
         'ASCII sorting is performed for names')
 
+    parser.epilog = """Filters using regular expressions are case-sensitive
+    by default. Append "(?i)" to a pattern to make it case-insensitive."""
+
     seq_mods = parser.add_argument_group("Sequence Modificaton")
     seq_mods.add_argument('--apply-function', type=module_function,
             metavar='/path/to/module.py:function_name[:parameter]',
@@ -51,7 +55,11 @@ def add_options(parser):
             action=partial_action(transform.multi_cut_sequences, 'slices'),
             help="""Keep only the residues within the 1-indexed start and end
             positions specified, : separated. Includes last item. Start or end
-            can be left unspecified to indicate start/end of sequence.""")
+            can be left unspecified to indicate start/end of sequence. A
+            negative start may be provided to indicate an offset from the end
+            of the sequence. Note that to prevent negative numbers being
+            interpreted as flags, this should be written with an equals
+            sign between `--cut` and the argument, e.g.: `--cut=-10:`""")
     seq_mods.add_argument('--relative-to', dest='cut_relative', metavar='ID',
             help="""Apply --cut relative to the indexes of non-gap residues in
             sequence identified by ID""")
@@ -111,32 +119,29 @@ def add_options(parser):
 
     seq_select = parser.add_argument_group("Record Selection")
 
-    seq_select.add_argument('--sample', metavar='N', dest='transforms', type=int,
-            action=partial_action(transform.sample, 'sample'),
-            help = """ Select a random sampling of sequences """)
     seq_select.add_argument('--deduplicate-sequences',
         action='store_const', const=None, default=False,
          dest='deduplicate_sequences', help='Remove any duplicate sequences '
          'by sequence content, keep the first instance seen')
     seq_select.add_argument('--deduplicated-sequences-file', action='store',
         metavar='FILE', dest='deduplicate_sequences', default=False,
-        type=common.FileType('w'),
+        type=common.FileType('wt'),
         help='Write all of the deduplicated sequences to a file')
     seq_select.add_argument('--deduplicate-taxa',
             action=partial_action(transform.deduplicate_taxa),
             dest='transforms', help="""Remove any duplicate sequences by ID,
             keep the first instance seen""")
     seq_select.add_argument('--exclude-from-file', metavar='FILE',
-            type=common.FileType('r'), help="""Filter sequences, removing
+            type=common.FileType('rt'), help="""Filter sequences, removing
             those sequence IDs in the specified file""", dest='transforms',
             action=partial_action(transform.exclude_from_file, 'handle'))
     seq_select.add_argument('--include-from-file', metavar='FILE',
-            type=common.FileType('r'), help="""Filter sequences, keeping only
+            type=common.FileType('rt'), help="""Filter sequences, keeping only
             those sequence IDs in the specified file""", dest='transforms',
             action=partial_action(transform.include_from_file, 'handle'))
-    seq_select.add_argument('--head', metavar='N', dest='transforms', type=int,
+    seq_select.add_argument('--head', metavar='N', dest='transforms',
             action=partial_action(transform.head, 'head'), help="""Trim
-            down to top N sequences""")
+            down to top N sequences. With the leading `-', print all but the last N sequences.""")
     seq_select.add_argument('--max-length', dest='transforms', metavar='N',
             action=partial_action(transform.max_length_discard, 'max_length'),
             type=int, help="""Discard any sequences beyond the specified
@@ -152,28 +157,33 @@ def add_options(parser):
                 than the specified minimum length, excluding gaps. This
                 operation occurs *before* cut and squeeze.""",
                 dest='transforms')
-    seq_select.add_argument('--pattern-include', metavar='regex',
+    seq_select.add_argument('--pattern-include', metavar='REGEX',
             action=partial_action(transform.name_include, 'filter_regex'),
             dest='transforms', help="""Filter the sequences by regular
-            expression in name""")
-    seq_select.add_argument('--pattern-exclude', metavar='regex',
+            expression in ID or description""")
+    seq_select.add_argument('--pattern-exclude', metavar='REGEX',
             action=partial_action(transform.name_exclude, 'filter_regex'),
             dest='transforms', help="""Filter the sequences by regular
-            expression in name""")
+            expression in ID or description""")
     seq_select.add_argument('--prune-empty',
             action=partial_action(transform.prune_empty), dest='transforms',
             help="Prune sequences containing only gaps ('-')")
-    seq_select.add_argument('--seq-pattern-include', metavar='regex',
+    seq_select.add_argument('--sample', metavar='N', dest='transforms', type=int,
+            action=partial_action(transform.sample, 'k'),
+            help = """ Select a random sampling of sequences """)
+    seq_select.add_argument('--sample-seed', metavar='N', type=int,
+            help = """Set random seed for sampling of sequences""")
+    seq_select.add_argument('--seq-pattern-include', metavar='REGEX',
             action=partial_action(transform.seq_include, 'filter_regex'),
             dest='transforms', help="""Filter the sequences by regular
             expression in sequence""")
-    seq_select.add_argument('--seq-pattern-exclude', metavar='regex',
+    seq_select.add_argument('--seq-pattern-exclude', metavar='REGEX',
             action=partial_action(transform.seq_exclude, 'filter_regex'),
             dest='transforms', help="""Filter the sequences by regular
             expression in sequence""")
-    seq_select.add_argument('--tail', metavar='N', dest='transforms', type=int,
+    seq_select.add_argument('--tail', metavar='N', dest='transforms',
             action=partial_action(transform.tail, 'tail'),
-        help='Trim down to bottom N sequences')
+        help="""Trim down to bottom N sequences.  Use +N to output sequences starting with the Nth.""")
 
     id_mods = parser.add_argument_group("Sequence ID Modification")
     id_mods.add_argument('--first-name',
@@ -192,15 +202,15 @@ def add_options(parser):
             action=partial_action(transform.name_replace, ('search_regex',
                 'replace_pattern')),
             dest='transforms', help="""Replace regex pattern "search_pattern"
-            with "replace_pattern" in sequence ID""")
+            with "replace_pattern" in sequence ID and description""")
     id_mods.add_argument('--strip-range', dest='transforms',
             action=partial_action(transform.strip_range), help="""Strip ranges
             from sequences IDs, matching </x-y>""")
 
     format_group = parser.add_argument_group('Format Options')
-    format_group.add_argument('--input-format', metavar='Format',
+    format_group.add_argument('--input-format', metavar='FORMAT',
             help="Input file format (default: determine from extension)")
-    format_group.add_argument('--output-format', metavar='Format',
+    format_group.add_argument('--output-format', metavar='FORMAT',
             help="Output file format (default: determine from extension)")
 
     parser.add_argument('--alphabet', choices=ALPHABETS,
@@ -214,7 +224,7 @@ def build_parser(parser):
     Add shared arguments to the convert or mogrify parser.
     """
     add_options(parser)
-    parser.add_argument('source_file', type=common.FileType('r'),
+    parser.add_argument('source_file', type=common.FileType('rt'),
                         help="Input sequence file")
     parser.add_argument('dest_file', help="Output file")
 
@@ -250,6 +260,10 @@ def transform_file(source_file, destination_file, arguments):
     # Apply all the transform functions in transforms
     if arguments.transforms:
 
+        # TODO: might be nice to somehow pass this directly into sample action
+        if arguments.sample_seed is not None:
+            random.seed(arguments.sample_seed)
+
         # Special case handling for --cut and --relative-to
         if arguments.cut_relative:
             for o, n in ((transform.multi_cut_sequences,
@@ -284,8 +298,7 @@ def transform_file(source_file, destination_file, arguments):
 
     # Only the fasta format is supported, as SeqIO.write does not have a 'wrap'
     # parameter.
-    if (arguments.line_wrap is not None and destination_file_type == 'fasta'
-            and source_file_type == 'fasta'):
+    if (arguments.line_wrap is not None and destination_file_type == 'fasta'):
         logging.info("Attempting to write fasta with %d line breaks.",
                 arguments.line_wrap)
 
@@ -319,7 +332,7 @@ def module_function(string):
 
     # Import the module
     module_vars = {}
-    execfile(module_path, module_vars)
+    exec(compile(open(module_path).read(), module_path, 'exec'), module_vars)
 
     try:
         function = module_vars[function_name]
@@ -336,6 +349,6 @@ def module_function(string):
 
 def action(arguments):
     with arguments.source_file as src, \
-            common.atomic_write(arguments.dest_file,
-                    file_factory=common.FileType('w')) as dest:
+            common.atomic_write(
+                arguments.dest_file, file_factory=common.FileType('wt')) as dest:
         transform_file(src, dest, arguments)
