@@ -1,7 +1,8 @@
-from cStringIO import StringIO
+from io import StringIO
 import os
 import os.path
 import logging
+import random
 import shlex
 import shutil
 import sys
@@ -15,29 +16,29 @@ from seqmagick.scripts import cli
 d = os.path.dirname(__file__)
 data_dir = os.path.join(d, "data")
 
+
 def p(*args):
     return os.path.join(data_dir, *args)
 
+
 class CommandLineTestMixIn(object):
     in_suffix = ''
     out_suffix = ''
 
     def setUp(self):
         self.input_file = tempfile.NamedTemporaryFile(suffix=self.in_suffix)
-        with open(self.input_path) as fp:
-            shutil.copyfileobj(fp, self.input_file)
-        self.input_file.flush()
+        shutil.copy(self.input_path, self.input_file.name)
         with tempfile.NamedTemporaryFile(suffix=self.out_suffix) as tf:
             self.output_file = tf.name
 
     def test_run(self):
-        command = self.command.format(input=self.input_file.name,
-                output=self.output_file)
+        command = self.command.format(
+            input=self.input_file.name, output=self.output_file)
         cli.main(shlex.split(command))
 
-        with FileType('r')(self.output_file) as fp:
+        with FileType('rt')(self.output_file) as fp:
             actual = fp.read()
-        with FileType('r')(self.expected_path) as fp:
+        with FileType('rt')(self.expected_path) as fp:
             expected = fp.read()
         self.assertEqual(expected, actual)
 
@@ -46,6 +47,7 @@ class CommandLineTestMixIn(object):
         if os.path.isfile(self.output_file):
             os.remove(self.output_file)
 
+
 class BasicConvertTestCase(CommandLineTestMixIn, unittest.TestCase):
     in_suffix = '.fasta'
     out_suffix = '.phy'
@@ -53,6 +55,8 @@ class BasicConvertTestCase(CommandLineTestMixIn, unittest.TestCase):
     expected_path = p('output2.phy')
     command = 'convert {input} {output}'
 
+
+@unittest.skipIf(sys.version_info.major == 3, 'bzip2 not supported')
 class BzipInputConvertTestCase(CommandLineTestMixIn, unittest.TestCase):
     in_suffix = '.fasta.bz2'
     out_suffix = '.phy'
@@ -60,6 +64,8 @@ class BzipInputConvertTestCase(CommandLineTestMixIn, unittest.TestCase):
     expected_path = p('output2.phy')
     command = 'convert {input} {output}'
 
+
+@unittest.skipIf(sys.version_info.major == 3, 'bzip2 not supported')
 class BzipOutputConvertTestCase(CommandLineTestMixIn, unittest.TestCase):
     in_suffix = '.fasta'
     out_suffix = '.phy.bz2'
@@ -67,6 +73,7 @@ class BzipOutputConvertTestCase(CommandLineTestMixIn, unittest.TestCase):
     expected_path = p('output2.phy')
     command = 'convert {input} {output}'
 
+
 class GzipInputConvertTestCase(CommandLineTestMixIn, unittest.TestCase):
     in_suffix = '.fasta.gz'
     out_suffix = '.phy'
@@ -74,6 +81,7 @@ class GzipInputConvertTestCase(CommandLineTestMixIn, unittest.TestCase):
     expected_path = p('output2.phy')
     command = 'convert {input} {output}'
 
+
 class GzipOutputConvertTestCase(CommandLineTestMixIn, unittest.TestCase):
     in_suffix = '.fasta'
     out_suffix = '.phy.gz'
@@ -81,12 +89,14 @@ class GzipOutputConvertTestCase(CommandLineTestMixIn, unittest.TestCase):
     expected_path = p('output2.phy')
     command = 'convert {input} {output}'
 
+
 class ConvertToNexusTestCase(CommandLineTestMixIn, unittest.TestCase):
     in_suffix = '.fasta'
     input_path = p('input2.fasta')
     expected_path = p('output2.nex')
     command = 'convert {input} {output} --output-format nexus --alphabet dna-ambiguous'
 
+
 class ConvertUngapCutTestCase(CommandLineTestMixIn, unittest.TestCase):
     in_suffix = '.fasta'
     out_suffix = '.fasta'
@@ -94,6 +104,7 @@ class ConvertUngapCutTestCase(CommandLineTestMixIn, unittest.TestCase):
     expected_path = p('output2_ungap_cut.fasta')
     command = 'convert --ungap --cut 1:3 --tail 2 {input} {output}'
 
+
 class ConvertToStdOutTestCase(unittest.TestCase):
 
     def setUp(self):
@@ -152,6 +163,23 @@ class TestTranslateAmbiguous(CommandLineTestMixIn, unittest.TestCase):
         super(TestTranslateAmbiguous, self).tearDown()
         logging.getLogger(None).setLevel(self.orig_level)
 
+class TestSample(CommandLineTestMixIn, unittest.TestCase):
+    in_suffix = '.fasta'
+    out_suffix = '.fasta'
+    input_path = p('input5.fasta')
+    expected_path = p('output5.fasta')
+    command = 'convert --sample 2 --sample-seed 0 {input} {output}'
+
+    def setUp(self):
+        super(TestSample, self).setUp()
+        self.orig_level = logging.getLogger(None).level
+        logging.getLogger(None).setLevel(logging.FATAL)
+        random.seed(1)
+
+    def tearDown(self):
+        super(TestSample, self).tearDown()
+        logging.getLogger(None).setLevel(self.orig_level)
+
 class TestStdin(TestTranslateAmbiguous, unittest.TestCase):
     command = 'convert --translate dna2protein - {output}'
 
@@ -165,6 +193,7 @@ class TestStdin(TestTranslateAmbiguous, unittest.TestCase):
         sys.stdin.close()
         sys.stdin = self.orig_stdin
 
+
 class TestConvertFromStdin(TestTranslateAmbiguous, unittest.TestCase):
     command = 'convert --translate dna2protein - {output}'
 
